/*
 * fr-command-lrzip.h
 *
 *  Created on: 10.04.2010
 *      Author: Alexander Saprykin
 */

#ifndef FRCOMMANDLRZIP_H_
#define FRCOMMANDLRZIP_H_

#include "fr-command.h"

G_DECLARE_FINAL_TYPE (FrCommandLrzip, fr_command_lrzip, FR, COMMAND_LRZIP, FrCommand)

#endif /* FRCOMMANDLRZIP_H_ */
